/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - Jeff Briggs, Henry Hughes, Ryan Morse
 *******************************************************************************/

package org.eclipse.linuxtools.systemtap.ui.graphingapi.ui.wizards.graph;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.linuxtools.internal.systemtap.ui.graphingapi.ui.Localization;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.forms.widgets.ColumnLayout;




public class SelectGraphWizardPage extends WizardPage implements Listener {
	public SelectGraphWizardPage() {
		super("selectGraph");
		setTitle(Localization.getString("SelectGraphWizardPage.SelectGraph"));
		btnGraphs = null;
	}
	
	public void createControl(Composite parent) {
		wizard = (SelectGraphWizard)getWizard();
		
		Composite comp = new Composite(parent, SWT.NULL);
		comp.setLayout(new FormLayout());
		FormData data1 = new FormData();
		data1.left = new FormAttachment(0, 0);
		data1.top = new FormAttachment(0, 0);
		data1.right = new FormAttachment(40, 0);
		data1.bottom = new FormAttachment(100, 0);
		
		Composite cmpGraphOpts = new Composite(comp, SWT.NONE);
		cmpGraphOpts.setLayoutData(data1);
		ColumnLayout colLayout = new ColumnLayout();
		colLayout.maxNumColumns = 1;
		cmpGraphOpts.setLayout(colLayout);
		
		String[] graphIDs = GraphFactory.getAvailableGraphs(wizard.model.getDataSet());
		btnGraphs = new Button[graphIDs.length];
		for(int i=0; i<btnGraphs.length; i++) {
			btnGraphs[i] = new Button(cmpGraphOpts, SWT.NONE);
			//btnGraphs[i].setText(GraphTypeConstants.getGraphName(graphIDs[i]));
			btnGraphs[i].setImage(GraphFactory.getGraphImage(graphIDs[i]));
			btnGraphs[i].addListener(SWT.Selection, this);
			btnGraphs[i].setData(graphIDs[i]);
		}
		
		FormData data2 = new FormData();
		data2.left = new FormAttachment(cmpGraphOpts);
		data2.top = new FormAttachment(0, 0);
		data2.right = new FormAttachment(100, 0);
		data2.bottom = new FormAttachment(100, 0);
		
		lblDesc = new Label(comp, SWT.WRAP);
		lblDesc.setLayoutData(data2);
		setControl(comp);
	}
	
	public void handleEvent(Event event) {
		if(event.widget instanceof Button) {
			Button target = (Button)event.widget;

			for(int i=0; i<btnGraphs.length; i++)
				if(target == btnGraphs[i]) {
					lblDesc.setText(GraphFactory.getGraphName(btnGraphs[i].getData().toString()) + "\n\n" +
									GraphFactory.getGraphDescription(btnGraphs[i].getData().toString()));
					saveDataToModel(btnGraphs[i].getData().toString());
					wizard.getContainer().updateButtons();
				}
		}
	}
	
	@Override
	public IWizardPage getNextPage() {    		
		return wizard.selectSeriesPage;
	}

	@Override
	public boolean canFlipToNextPage() {
		return wizard.model.isGraphSet();
	}
	
	private void saveDataToModel(String selected) {
		GraphModel model = wizard.model;
		model.setGraph(selected);
		wizard.selectSeriesPage.setKeyEnablement(
				GraphFactory.isKeyRequired(model.getGraphID(), model.getDataSet()), 
				GraphFactory.isKeyOptional(model.getGraphID(), model.getDataSet()));
	}
	
	@Override
	public void dispose() {
		super.dispose();
		if(null != btnGraphs)
			for(int i=0; i<btnGraphs.length; i++)
				btnGraphs[i] = null;
		btnGraphs = null;
		lblDesc = null;
	}
	
	private Button[] btnGraphs;
	private Label lblDesc;
	private SelectGraphWizard wizard;
}